import torch
from data_loading import load_and_preprocess, save_segmentation_nifti
from predict_patient import predict_patient_3D_net
import imp
from utils import postprocess_prediction
#from predict_patient import predict_patient_3D_net_tiled


def run(mri_fname, output_fname, params_file, config_file, device=0, postprocess=True):
    cf = imp.load_source('cf', config_file)
    cf = cf.config()

    net, _ = cf.get_network(cf.val_use_train_mode, params_file)
    if device == "cpu":
        net = net.cpu()
    else:
        net.cuda(device)

    data, data_dict = load_and_preprocess(mri_fname)

    seg, _, _, _ = predict_patient_3D_net(net, cf.preprocess(data.transpose(0, 3, 2, 1)[:, :, ::-1, ::-1]), cf.val_do_mirroring, cf.val_num_repeats, cf.val_batch_size,
                                 cf.net_input_must_be_divisible_by, cf.val_min_size, device, cf.da_mirror_axes)

    #seg, _= predict_patient_3D_net_tiled(net, cf.preprocess(data.transpose(0, 3, 2, 1)[:, :, ::-1, ::-1]), cf.val_num_repeats, cf.INPUT_PATCH_SIZE,
    #                                            cf.val_batch_size, device, True, 2, cf.val_do_mirroring,
    #                                            cf.da_mirror_axes)
    seg = seg[:, ::-1, ::-1].transpose(2, 1, 0)

    if postprocess:
        seg = postprocess_prediction(seg)

    save_segmentation_nifti(seg, data_dict, output_fname)

if __name__ == "__main__":
    params_file = "model_final.model"
    config_file = "basic_config_just_like_braintumor.py"

    device = 0

    run("0001179516_20120615_T1.nii", "0001179516_20120615_T1_mask_bet.nii.gz", params_file, config_file, device, True)
